package edu.toronto.cs.algebra;

import edu.toronto.cs.util.*;
import edu.toronto.cs.util.IntIterator.*;
import java.util.*;
import java.io.PrintWriter;

/****
 **** A finite lattice
 ****/
public class BooleanLattice implements Algebra
{

  public static final int MEET = 0;
  public static final int JOIN = 1;
  public static final int NEG = 2;
  public static final int BELOW = 3;
  public static final int ABOVE = 4;
  public static final int EQUAL = 5;

  public static final int FALSE = 0;
  public static final int TRUE = 1;
  
  public BooleanLattice ()
  {
  }

  public int binOp (int l, int r, int opcode) throws NotAnOperationException
  {
    switch (opcode)
      {
      case MEET:
	return l & r;
      case JOIN:
	return l | r;
      case BELOW:
	return l <= r ? TRUE : FALSE;
      case ABOVE:
	return l >= r ? TRUE : FALSE;
      case EQUAL:
	return l == r ? TRUE : FALSE;
      default:
	throw new NotAnOperationException ("Unsupported opcode: " + opcode);
      }
  }

  public int unOp (int l, int opcode) throws NotAnOperationException
  {
    switch (opcode)
      {
      case NEG:
	return l == TRUE ? FALSE : TRUE;
      default:
	throw new NotAnOperationException ("Unsupported opcode: " + opcode);
      }
  }
  
  public int plus (int l, int r) throws NotInCarrierSetException
  {
    return l | r;
  }
  
  public int idplusIdx ()
  {
    return FALSE;
  }
  
  
  public int times (int l, int r) throws NotInCarrierSetException
  {
    return l & r;
  }
  
  public int idtimesIdx ()
  {
    return TRUE;
  }
  
  public int size ()
  {
    return 2;
  }
  
  public IntIterator carrierSet ()
  {
    return new RangeIterator (0, 2);
  }
  
  // -- eq, geq, leq
  public boolean eq (int a, int b)
  {
    return a == b;
  }
  
  // -- computes a under b
  public boolean leq (int a, int b)
  {
    return a <= b;
  }
  public boolean geq (int a, int b)
  {
    return a >= b;
  }
  public boolean isComparable (int a, int b)
  {
    return true;
  }
  
    
  public int getElementIdxByKey (Object key)
  {
    if (key == null) return Algebra.NO_VALUE;

    if (key.equals ("T")) return TRUE;
    if (key.equals ("F")) return FALSE;
    
    return Algebra.NO_VALUE;
  }
  
  public Object getElementKeyByIdx (int idx)
  {
    if (idx == TRUE) return "T";
    else return "F";
  }

}



