package edu.toronto.cs.ctl;

import edu.toronto.cs.algebra.*;
import java.util.*;

public class CTLDirectSimplifier implements CTLSimplifier
{
  // -- this should be a generic simplifier that expands implications and 
  // -- so on.
  CTLSimplifier otherSimplifier = new CTLEXEUEGSimplifier ();
  
  public CTLNode simplify (CTLNode ctl, FiniteLattice lattice)
  {
    if (ctl instanceof CTLAUiNode)
      return simplifyAUi ((CTLAUiNode)ctl);
    else if (ctl instanceof CTLAFNode)
      return simplifyAF ((CTLAFNode)ctl, lattice);    
    // -- things that the model checker should handle directly
    else if (ctl instanceof CTLAXNode ||
	     ctl instanceof CTLAGNode ||
	     ctl instanceof CTLAUNode)
      return ctl;
    else
      return otherSimplifier.simplify (ctl, lattice);
    
  }
  
  
  CTLNode simplifyAUi (CTLAUiNode ctl)
  {
    CTLNode f = ctl.getLeft ();
    CTLNode p = ctl.getRight ();
    
    int i = ctl.getI ();
    
    if (i == 0) return p;
    
    CTLAUiNode afup = f.au (i - 1, p);
    return p.or (f.and (afup.ax ()));
  }
  
  CTLNode simplifyAF (CTLAFNode ctl, FiniteLattice lattice)
  {
    CTLNode f = ctl.getRight ();
    CTLLatticeNode top = 
      CTLFactory.createCTLLatticeNode (lattice.idtimesIdx (), lattice);
    return CTLFactory.createCTLAUNode (top, f, ctl.getFairness ());
    //    return top.au (f);
  }
  
}
