package edu.toronto.cs.experiments;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.smv.*;
import edu.toronto.cs.mdd.*;
import edu.toronto.cs.util.*;

import java.util.*;
import java.io.*;


public class SMVBddNoiseModule extends SMVBddModule
{
  String bddFileName;
  int noiseLevel;
  
  public SMVBddNoiseModule (String[] _vars, String _bddFileName, 
			    int _noiseLevel)
  {
    super (_vars, _bddFileName);
    noiseLevel = _noiseLevel;
  }

  public MvSet computeTrans ()
  {
    MvSet trans = super.computeTrans ();
    return introduceMvNoise (trans);
  }
  

  /*** here we get a random value that is not \top or \bot ***/
  public int getRandomValue ()
  {
    int lo = top > bot ? bot : top;
    int hi = top > bot ? top : bot;

    // -- we need to exclude the lowest number, highest is excluded by itself
    lo++;
    
    int val = Chaos.inRange (lo, hi);
    // System.out.println ("Decided on a val: " + val);
    return val;
  }


  // -- introduce mv-noise into a decision diagram
  public  MvSet introduceMvNoise (MvSet mvSet)
  {
    int top = lattice.idtimesIdx ();
    int bot = lattice.idplusIdx ();
    
    ChaosIterator varDomain = new UniformChaosIterator (new int[] {top, bot});

    MDDNode node = ((MDDMvSet)mvSet).getMddNode ();
    MDDManager manager 
      = ((MDDMvSet.MDDMvSetFactory)ddManager).getMddManager ();
    
    // -- given a decision diagram, and an FTO with the middle value M
    // -- we want to map some \top cubes to the interval (\top, M]
    // -- and some \bot cubes to the interval [M, \top)
    MDDNode positiveCubes = manager.getZero ();
    MDDNode negativeCubes = manager.getZero ();
    
    // -- find cubes we will change
    for (int i = 0; i < noiseLevel; i++)
      {
	// -- get a random point
	MDDPoint point = manager.getRandomPoint (node, varDomain);
	if (point.getValue () == top)
	  {

	    // -- got a positive cube that we want to augment
	    
	    // -- get a random negative value i.e. in interval [M, \bot)
	    // -- since we will negate everything later on anyways
	    int val = getRandomValue ();

	    MDDNode pointNode = point.getNode ();
	    
	    // -- set the value
	    pointNode = manager.setEqualTo (pointNode, val);
	    
	    
	    // -- add to the rest of our cubes
	    positiveCubes = manager.Apply (positiveCubes, pointNode, 
					   FiniteLattice.JOIN);
	  }
	else
	  {
	    int val = getRandomValue ();
	    MDDNode pointNode = point.getNode ();
	    pointNode = manager.setEqualTo (pointNode, val);
	    
	    negativeCubes = manager.Apply (negativeCubes, pointNode,
					   FiniteLattice.JOIN);
	  }
	
      }
    

    MDDNode result = node;

    System.out.println ("Merging all negative cubes...");
    // -- change all of the negative cubes    
    result = manager.Apply (result, negativeCubes, 
			    FiniteLattice.JOIN);
    
    System.out.println ("Merging all positive cubes....");
    // -- change all positive cubes
    result = manager.Apply 
      (result, manager.Apply (positiveCubes, FiniteLattice.NEG),
       FiniteLattice.MEET);
    return ((MDDMvSet.MDDMvSetFactory)ddManager).createMvSet (result);
  }

  
  
  public static SMVModule initModule (String[] args)
  {
    Chaos.seed (0);
    /***
	args [0] -- lattice name
	args [1] -- prop ordering file
	args [2] -- bdd file
	args [3] -- noiseLevel
    ***/
    String latticeName = args [0];
    String propOrderFileName = args [1];
    String bddFileName = args [2];

    int noiseLevel = 0;
    try {
      noiseLevel = Integer.parseInt (args [3]);
    }
    catch (Exception ex){
      Assert.assert (ex);
    }
    
    
    String[] vars = SMVBddModule.computeVars (propOrderFileName);
    SMVBddModule module = 
      new SMVBddNoiseModule (vars, bddFileName, noiseLevel);
    module.setLattice (Lattices.getLatticeByName (latticeName));
    return module;
  }
  

  
}
