package edu.toronto.cs.gui;

import edu.toronto.cs.modelchecker.*;

import java.util.*;
import java.io.*;
import javax.swing.*;
//import javax.swing.event.*;
import javax.swing.filechooser.*;

import java.awt.*;
import java.awt.event.*;

/**
 ** The "Open BDD" dialog window.
 **/
public class OpenBDDPanel extends JDialog
{
  FilePanel bddfile, propfile, initfile, enumfile;

  // makes all the file names the same as the bdd file
  JCheckBox same;

  // the done and cancel buttons
  JButton done, cancel;

  // selected files
  String [] files = null;

  public OpenBDDPanel (JFileChooser filechooser)
  {
    super ((Frame) null, "Open BDD Model", true);

    bddfile = new FilePanel ("BDD file:", filechooser);
    bddfile.addActionListener (new ActionListener ()
			       {
				 public void actionPerformed (ActionEvent e)
				   {
				     if (same.getSelectedObjects () != null)
				       makeSameFileNames ();
				   }
			       });
    propfile = new FilePanel ("Prop ordering file:", filechooser);
    initfile = new FilePanel ("Initial state file:", filechooser);
    enumfile = new FilePanel ("Enumerated types file:", filechooser);
    same = new JCheckBox ("Make all the filenames same as BDD", false);
    same.addActionListener (new ActionListener ()
			    {
			      public void actionPerformed (ActionEvent e)
				{
				  boolean disabled = !propfile.isEnabled ();

				  // switch between enabled/disabled
				  GUIutil.setEnabled (propfile, disabled);
				  GUIutil.setEnabled (initfile, disabled);
				  GUIutil.setEnabled (enumfile, disabled);
				  
				  if (!disabled) makeSameFileNames ();
				}
			    });

    // the two buttons at the bottom of the screen - done and cancel
    JPanel donecancel = new JPanel ();
    donecancel.setLayout (new BoxLayout (donecancel, BoxLayout.X_AXIS));
    done = new JButton ("Done");
    done.setMnemonic (KeyEvent.VK_D);
    done.addActionListener (new ActionListener ()
			    {
			      public void actionPerformed (ActionEvent e)
				{			  
				  files = new String [4];
				  
				  files[0] = propfile.getFileName ();
				  files[1] = bddfile.getFileName ();
				  files[2] = initfile.getFileName ();
				  files[3] = enumfile.getFileName ();

				  setVisible (false);
				}
			    });
    cancel = new JButton ("Cancel");
    cancel.setMnemonic (KeyEvent.VK_C);
    cancel.addActionListener (new ActionListener ()
			      {
				public void actionPerformed (ActionEvent e)
				  {				  
				    files = null;
				    setVisible (false);
				  }
			      });
    donecancel.add (done);
    donecancel.add (Box.createGlue ());
    donecancel.add (cancel);

    // make the layout a bit nicer (more space between everything)
    JPanel all = new JPanel ();
    all.setLayout (new BoxLayout (all, BoxLayout.Y_AXIS));
    all.setBorder (StandardFiller.makeWideEmptyBorder ());

    all.add (bddfile);
    all.add (StandardFiller.makeVstrut ());
    all.add (propfile);
    all.add (StandardFiller.makeVstrut ());
    all.add (initfile);
    all.add (StandardFiller.makeVstrut ());
    all.add (enumfile);
    all.add (StandardFiller.makeVstrut ());
    all.add (same);
    all.add (StandardFiller.makeVstrut ());
    all.add (donecancel);

    // set alignment
    GUIutil.alignAllX (all.getComponents (), Component.LEFT_ALIGNMENT);

    getContentPane ().add (all);
  }

  /**
   ** Get the selected file names.
   ** @return the selected file names or null (if 'Cancel' is pressed).
   **/
  String [] getFileNames ()
  {
    return files;
  }
  
  /**
   ** Make all the file names the same.
   **/
  void makeSameFileNames ()
  {
    String name = bddfile.getFileName ();
    
    if (name.endsWith (".bdd"))
      name = name.substring (0, name.length () - 4);
    
    propfile.setFileName (name + ".ord");
    initfile.setFileName (name + ".init");
    enumfile.setFileName (name + ".evars");
  }

}

/**
 ** A panel for inputting a file.
 **/
class FilePanel extends JPanel
{
  JLabel label;
  JTextField filename;
  JButton browse;

  JFileChooser filechooser;

  FilePanel (String text, JFileChooser fc)
  {
    super ();

    filechooser = fc;
    // set up the components
    label = new JLabel (text);
    filename = new JTextField (30);
    browse = new JButton ("Browse...");

    // set up the action listener for the button
    browse.addActionListener
      (new ActionListener ()
       {
	 public void actionPerformed (ActionEvent e)
	   {
	     System.out.println ("Got action: " + e);
	     // get the file  
	     filechooser.setDialogTitle (label.getText ());
	     int result = filechooser.showOpenDialog (null);
	     if (result == JFileChooser.APPROVE_OPTION)
	       filename.setText (filechooser.getSelectedFile ().
				 getAbsolutePath ());
	   }
       });
    
    // set the layout and put all of it together
    setLayout (new BoxLayout (this, BoxLayout.X_AXIS));
    add (label);
    add (StandardFiller.makeHstrut ());
    add (filename);
    add (StandardFiller.makeHstrut ());
    add (browse);
  }

  void setFileName (String name)
  {
    filename.setText (name);
  }

  String getFileName ()
  {
    return filename.getText ();
  }
  
  void addActionListener (ActionListener l)
  {
    filename.addActionListener (l);
  }
}
