package edu.toronto.cs.keg;

import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.heuristics.*;
import edu.toronto.cs.modelchecker.*;

/**
 ** This is the interface to the expands the states and/or formulas in
 ** order to produce a counter-example.
 **/
public interface KEGExpander
{

  /**
   ** Get the heuristics.
   ** @return value of heuristics.
   **/
  HeuristicAnalyser getHeuristics ();
    
  /**
   ** Set the heuristics.
   ** @param h -- heuristics to be used.
   **/
  void setHeuristics(HeuristicAnalyser h);
  
  /**
   ** Attempts to expand the state or the formula associated with the
   ** state in order to provide a counter-example.
   **
   ** @param state -- a node in the state graph that contains the state
   ** to be considered.
   **
   ** @param formula -- a node in the formula graph that contains the
   ** formula to be considered.
   **
   ** @param mc -- the model checker to be used.
   **
   ** @return -- true if the expansion was successful; false otherwise.
   **/
  boolean expand (DGNode state, DGNode formula, MvSetModelChecker mc);

}
