package edu.toronto.cs.keg;

import java.util.*; // We all love java.util
import java.io.*;

import edu.toronto.cs.ctl.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.xkripke.XKripke.*;
import edu.toronto.cs.prettygraph.*;
import edu.toronto.cs.heuristics.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.util.userprompt.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.modelchecker.*;

import edu.toronto.cs.algebra.*;
import edu.toronto.cs.smv.*;
import java.lang.reflect.*;

import edu.toronto.cs.tlq.UpSetLattice;


class KEGTest
{
  XKripke xk;
  MvSetModelChecker modelChecker;
  int numctl = 0;
  Map options; // -- the various options for the KEG
  MvState initState = null;
  EnumeratedTypes enumTypes = EnumeratedTypes.EMPTY_TYPES;
  
  public KEGTest (Map opts)
  {
    options = new HashMap (opts);
  }

  public String getAutoName ()
  {
    return "" + options.get ("prefix") + numctl + ".daVinci";
  }

  public void saveKegToFile (DGraph graph, String filename, Map colourMap)
  {  
    DGraph daVinci = 
      PrettyGraphKegVisitor.getPrettyRep 
      (graph, modelChecker.getPropOrdering (), colourMap);
    StringBuffer sb = new StringBuffer ();
    DVGenNodeVisitor dv = new DVGenNodeVisitor (new DVGenEdgeVisitor ());
    for (Iterator it = DGDepthTraverse.getIterator (daVinci); it.hasNext ();)
      {
	dv.visit ((DGNode) it.next (), null);
      }
  
    sb = dv.getSB ();
    saveToFile (sb, filename);
  }

  public void saveToFile (StringBuffer sb, String filename)
  {
    try 
      {
	FileWriter daVinciFile;
	daVinciFile = new FileWriter (filename);
	daVinciFile.write (StringUtil.doEscapes(sb.toString ()));
	daVinciFile.close ();
      }
    catch (Exception e)
      {
	Assert.assert (e);
      }
  }

  /**
   ** Creates a model-checker, retrieves the kripke structure from the
   ** file 'filename' and feeds it into the model-checker for further
   ** analysys.
   **
   ** @param filename -- the name of hte file containing the XKripke
   ** structure
   **
   ** @return -- the model-checker initialized with the XKripke
   ** structure from the specified file
   **/
  public MvSetModelChecker initMC (String filename)
  {
    try 
      {
	xk = XKripkeFactory.parse (filename);

	if (((Boolean) options.get ("statenames")).booleanValue ())
	  xk.labeliseStates ();

	/*
	DGraph pg = PrettyGraphXKVisitor.getPrettyXKGraph (xk);
	DVGenNodeVisitor dvg = 
	  new DVGenNodeVisitor (new DVGenEdgeVisitor ());
	for (Iterator it = DGDepthTraverse.getIterator (pg); it.hasNext ();)
	  {
	    dvg.visit ((DGNode)it.next ());
	  }
	StringBuffer modeldv = dvg.getSB ();
	System.out.println ("Please enter filename for model dv rep:");
	saveToFile (modeldv, getFileNameFrom (System.in));
	*/
	MvSetModelChecker mc = new MvSetModelChecker (xk);
	return mc;
      }
    catch (Exception e)
      {
	Assert.assert (e);
      }
    return null;
  }

  public MvSetModelChecker initMCSMV ()
  {
    try {
      return initMCUsingSMVModule ();
    }
    catch (Exception ex)  {
      ex.printStackTrace ();
      throw new RuntimeException ("Some error or other");
    }
    
  }

  public MvSetModelChecker initMCUsingSMVModule () throws Exception
  {
    // -- here we initialize the model checker using SMV module from 
    // -- command line argument. 
    // -- we need to initialize
    // -- 1) Lattice
    // -- 2) Prop ordering
    // -- 3) Transition relation
    // -- 4) Initial state
    
    List smvModelCmdLine = (List)options.get ("input");
    
    // -- extract smvModule name and its arguments
    String smvModuleName = (String)smvModelCmdLine.get (0);
    String[] smvModuleArgs = 
      (String[])smvModelCmdLine.subList (1, smvModelCmdLine.size ()).
      toArray (new String [smvModelCmdLine.size () - 1]);
    
    SMVModule smvModule = loadSMVModule (smvModuleName, smvModuleArgs);

    FiniteLattice lattice = smvModule.getLattice ();
    

    PropOrderIf propOrder = new AlternatePropOrder (smvModule.getVars ());
    smvModule.setVarOrdering (propOrder);

    MvSetFactory ddManager = 
      MDDMvSet.newMvSetFactory (lattice, propOrder.size ());
    
    smvModule.setMvSetFactory (ddManager);

    // -- set init state
    initState = new MvState (propOrder.size (), lattice);
    initState = smvModule.initializeInitState (initState);

    enumTypes = smvModule.getEnumTypes ();

    // -- compute transition relation
    return new MvSetModelChecker (smvModule.computeTrans (), propOrder);
  }

  
  private SMVModule loadSMVModule (String className, String[] args) 
    throws Exception
  {
    Class clazz = Class.forName (className);
    Method initMethod = clazz.getMethod ("initModule", 
					 new Class[]{String[].class});
    return (SMVModule)initMethod.invoke (clazz, new Object[]{args});
  }

  
  /**
   ** Reads CTL from an InputStream. If there is nothing ("") in the
   ** InputStream, returns null.
   **
   ** @param sourse -- where to read the CTL from
   **
   ** @return -- the CTL read or null
   **/
  public CTLNode getCtlFrom (InputStream source)
  {
    // -- prepare to read from stdin
    NiceReader in = 
      new NiceReader (source);
    String ctlStr = in.readLine ();
    if (!ctlStr.equals (""))
      {
	BinaryTreeToCTLConverter.AssignLattice
	  (modelChecker.getLattice ());

	// -- create a macro manager
	CTLMacroManager macroManager = 
	  new CTLMacroManager (modelChecker.getLattice ());

	if (modelChecker.getLattice () instanceof UpSetLattice)
	  macroManager.addExpander 
	    (new PlaceholderManager2 
	     ((UpSetLattice)modelChecker.getLattice (), enumTypes));

	if (enumTypes != null && enumTypes != EnumeratedTypes.EMPTY_TYPES)
	  macroManager.addExpander (new SMVEnumTypeExpander (enumTypes));
	    
	
	CTLNode ctl = BinaryTreeToCTLConverter.convertToCTL (ctlStr);

	ctl = macroManager.expandMacro (ctl);
	ctl = macroManager.expandMacro (ctl);
	return ctl;
      }
    else
      return null;
  }

  /**
   ** Asks user to input the CTL property to check.
   **
   ** @return -- the user input
   **/
  public Object getUserInput () // So we can eventually have UINode
  {
    CTLNode ctl = null;
    System.out.println ("CTL, please:");
    ctl = getCtlFrom (System.in);
    return ctl;
  }

  /**
   ** The main driver for the keg test.
   **/
  public static void main (String args [])
  {
    // -- parse the command line arguments
    Map opts = KEGClapi.parseOpts (args);

    KEGTest k2 = new KEGTest (opts);
    k2.run ();
  }
  
  /**
   ** Runs the KEG with the parameters that were previously set
   ** (usually in main).
   **/
  public void run ()
  {

    if (((Boolean)options.get ("smv")).booleanValue ())
      modelChecker = initMCSMV ();
    else
      {
	// -- get the file name from the options
	// -- note that there is a List of input files, for now we only
	// consider the first element in that list
	String filename = (String) ((List) options.get ("input")).get (0);
	
	modelChecker = initMC (filename);
	initState = getXKInitState ();
      }
    
    CTLNode ctl;
    int numctl=0;
    while (true)
      {
	Object o = getUserInput ();
	
	if (o == null)
	  break; // stop;
	else if (o instanceof CTLNode)
	  doCtl ((CTLNode)o);
	
	
      }
    
  }
  
  /**
   ** Given a CTL formula (CTLNode) creates a counter-example for it
   ** and save it to a file.
   **/
  public void doCtl (CTLNode ctl)
  {
    numctl++; // # of properties that have been checked

    // get the graphs to store the counter-examples
    DGraph KGraph = initKegStruct (ctl, initState);
    KEGStateNode initNode = 
      (KEGStateNode)KGraph.getRootNodes ().iterator (). next ();
    KEGCtlNode initCtlNode = 
      (KEGCtlNode)initNode.getCTLGraph ().getRootNodes ().iterator ().next ();
    Set selection;
    runOnCtl (initNode, initCtlNode);

    // get the filename from the user
    NiceReader reader = new NiceReader (System.in);
    System.out.println 
      ("Please enter filename; hit enter for automatic. ("
       + getAutoName () + ")");
    String filename = reader.readLine ();
    if (filename.equals (""))
      filename = getAutoName (); // use default filename
    saveKegToFile (KGraph, filename, PrettyConstants.singletonMap);
  }

  Set pickAll (Collection collection)
  {
    Set set = new HashSet ();
    for (Iterator it = collection.iterator (); it.hasNext ();)
      set.add (it.next ());
    return set;
  }
  
  /**
   ** Expands the counter-example graph.
   **/
  public void runOnCtl (KEGStateNode initNode, KEGCtlNode initCtlNode)
  {
    KEGExpander ctlEUiExp = new KEEUiSimplifier ();
    
    KEGExpander ctlexp = new KECTLSimplifier 
      (new SimpleHeuristicAnalyser ());
    KEGExpander statexp = new KEMDDStateGenerator ();
    Set selection;
    if (ctlEUiExp.expand (initNode, initCtlNode, modelChecker))
      // -- guaranteed to give one successor
      runOnCtl (initNode, 
		(KEGCtlNode)initCtlNode.
		getSuccessors ().iterator ().next ());  
    else if (ctlexp.expand (initNode, initCtlNode, modelChecker))
      {
	if (initCtlNode.getSuccessors ().size () > 1)
	  {
	    // -- XXX Arie: Modified to always pick all for now
// 	    selection =
// 	      UserPrompt.pickSome (initCtlNode.getSuccessors ());
	    selection = pickAll (initCtlNode.getSuccessors ());
	  }
	else
	  {
	    selection = new HashSet ();
	    selection.add ( initCtlNode.getSuccessors ().iterator ().next ());
	  }
	for (Iterator it = selection.iterator (); it.hasNext ();)
	  {
	    runOnCtl (initNode, (KEGCtlNode)it.next ());
	  }
	
      }
    else if (statexp.expand (initNode, initCtlNode,
			     modelChecker))
      {
	if (initNode.getSuccessors ().size () > 1)
	  { 
	    //oldinit = initNode;
	    // -- XXX Arie: Modified to always pick all for now	    
	    //selection = UserPrompt.pickSome (initNode.getSuccessors ());
	    selection = pickAll (initNode.getSuccessors ());
	  }
	else 
	  {
	    //oldinit = initNode;
	    selection = new HashSet ();
	    selection.add (initNode.getSuccessors ().iterator ().next ());  
	  }
	for (Iterator it = selection.iterator (); it.hasNext ();)
	  {
	    KEGStateNode nextstate = (KEGStateNode) it.next ();
	    KEGCtlNode nextctl = (KEGCtlNode)
	      nextstate.getCTLGraph ().getRootNodes ().iterator ().next ();
	    runOnCtl (nextstate, nextctl);
	  }
      }	      
  }
  
  public DGraph initKegStruct (CTLNode ctl, MvState initState)
  {
    System.out.println ("InintKegStruct: " + initState);
    DGraph stateGraph = new DGraph (new KEGStateNodeFactory ());
    addState (stateGraph, ((KEGStateNodeFactory)stateGraph.getFactory ())
	      .createNode (null, initState), ctl);
    return stateGraph;
    
  }
  
  public MvState getXKInitState ()
  {
    XKripkeState[] states = xk.getInitialStates ();
    XKripkeState initState = null;
    
    if (states.length < 1)
      {
	System.out.println ("NO initial states!");
	// return result;
      }
    else 
      {
	initState = states [0];
	if (states.length > 1) 
	  System.out.println ("WARNING: No unique initial state");
      }

    return new MvState (initState, modelChecker.getPropOrdering ());

  }
  
    
  public DGraph initKegStruct (CTLNode ctl)
  {
    DGraph stateGraph = new DGraph (new KEGStateNodeFactory ());
	    
    XKripkeState[] states = xk.getInitialStates ();
    XKripkeState initState = null;
    
    if (states.length < 1)
      {
	System.out.println ("NO initial states!");
	// return result;
      }
    else 
      {
	initState = states [0];
	if (states.length > 1) 
	  System.out.println ("WARNING: No unique initial state");
      }
    
    addState (stateGraph, ((KEGStateNodeFactory)stateGraph.getFactory ())
	      .createNode (null, new MvState 
		(initState, modelChecker.getPropOrdering ())), ctl);
    return stateGraph;
  }
  
  public void addState (DGraph stateGraph, KEGStateNode node, CTLNode ctl)
  {
    stateGraph.addRootNode (node);
    DGraph ctlGraph;
    setupCtl (node, ctl);
  }
  
  public void addState (KEGStateNode snode, KEGCtlNode cnode, 
			MvState addstate, CTLNode ctl)
  {
    KEGStateNode addnode = getNode (snode, addstate, 
				    (KEGStateNodeFactory)snode.getGraph ()
				    .getFactory ());
    snode.add (addnode);
    Set st = cnode.getStates ();
    st.add (addnode.getState ());
    cnode.makeTerminal (st);
    setupCtl (addnode, ctl);  
  }

  public KEGStateNode getNode (KEGStateNode parent, 
			       MvState addstate, KEGStateNodeFactory factory)
  {
    return factory.createNode 
      (parent, addstate);
    //modelChecker.getPropOrdering (), null);
  }

  public void addCtl (KEGStateNode snode, KEGCtlNode cnode, 
		      CTLNode ctlnode)
  {
    KEGCtlNode newKCtl = 
      ((KEGCtlNodeFactory)cnode.getFactory ()).createNode (null, ctlnode);
    cnode.add (newKCtl);
  }

  public void setupCtl (KEGStateNode addnode, CTLNode ctl)
  {
    DGraph ctlGraph = new DGraph (new KEGCtlNodeFactory ());
    addnode.setCTLGraph (ctlGraph);
    if (ctl != null)
      {
	KEGCtlNode initCtlNode;
	initCtlNode = 
	  ((KEGCtlNodeFactory)ctlGraph.getFactory ()).createNode (null, ctl);
	ctlGraph.addRootNode (initCtlNode);
      }
  }
  
}







