package edu.toronto.cs.lift;

import java.util.*;

import edu.toronto.cs.smv.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.util.*;


// XXX for testing only
import edu.toronto.cs.modelchecker.AlternatePropOrder;
import edu.toronto.cs.algebra.*;


/****
 **** Lift with ef feature, see exec.feat
 ****/

// -- XXX TODO
// -- XXX ef is a new numeric variable that we have to deal with 
// -- XXX just like 'floor' it ranges from 0 to topFloor
// -- XXX just like floor it needs equals, with things
// -- XXX TODO: Add priority_call into the picture
public class LiftREF extends SMVCombinedModule
{
  ButtonR[] liftBtns;
  int floors;

  int topFloor;
  int botFloor = 1;
  
  static final int DOOR = 0;
  static final int DIR = 1;

  int floorBaseIdx;
  int floorWidth;

  int efIdx;
  int efWidth;

  MvSet noCall;
  
  // -- XXX This is needed to reference the main module due to some
  // -- XXX technical problems which should be solved eventually
  MainLiftREF main;
  
  public LiftREF (String _name, int _floors, MainLiftREF _main)
  {
    super (_name);
    floors = _floors;
    topFloor = floors;

    floorBaseIdx = 2;
    floorWidth = computeFloorWidth (topFloor);
    efIdx = floorBaseIdx + floorWidth;
    // -- ef is the same type as floor
    efWidth = floorWidth;
    
    
    vars = new String [2 + floorWidth + efWidth];
    
    vars [DOOR] = "door";
    vars [DIR] = "dir";
    for (int i = floorBaseIdx; i < floorBaseIdx + floorWidth; i++)
      vars [i] = "floor" + (i - floorBaseIdx);
    for (int i = efIdx; i < efIdx + efWidth; i++)
      vars [i] = "ef" + (i - efIdx);

    // -- do the buttons
    liftBtns = new ButtonR [floors];
    for (int i = 1; i <= liftBtns.length; i++)
      liftBtns [i-1] = new ButtonR (getName () + 
				    SMVModule.VAR_SEP + "liftBut" + i);
    addSubModules (Arrays.asList (liftBtns));

    main = _main;
  }


  public void setNoCall (MvSet v)
  {
    noCall = v;
  }
  
  
  // -- compute number of bits required to represent v
  int computeFloorWidth (int v)
  {
    int width = 1;

    while ((v >> width) > 0) width++;
    return width;
  }
  

  public MvSet computeTrans ()
  {
    System.out.println ("Computing trans for " + getName ());

    MvSet result = computeButtons ().and 
      (computeDir ().and
       (computeDoor ().and
	(computeFloor ())));

    return result;
  }
  
  
  MvSet computeButtons ()
  {

    MvSet pressedCond = computeNatEquals (efIdx, efWidth, 0).not ().and
      (computeEfIsLandingCall ());

    MvSet result = mvTop;
    
    MvSet doorOpen = cur (DOOR, top);
    for (int i = 0; i < liftBtns.length; i++)
      {
	liftBtns [i].setReset (computeFloorEquals (i+1).and (doorOpen));
	
	// XXX Feature
	liftBtns [i].setPressedCond (pressedCond, bot);
	
	result = result.and (liftBtns [i].computeTrans ());
	if (i == 0 && result instanceof MDDMvSet)
	  ((MDDMvSet)result).writeToDaVinci ("ButtonR.daVinci", ordering);
      }

    return result;
  }

  
  MvSet computeDir ()
  {
    /*
    ** up is true, down is false **
    next(dir) := case
               idle : dir;
	       floor = topFloor : false;
	       floor = botFloor : true;
	       car_call = 0 & landing_call = 0 & dir = false : true;
	       car_call = 0 & landing_call = 0 & dir = true : false;
	       1  : dir;
	       esac;
    */

    // -- mvset for next(dir) = dir
    MvSet dirIsDir = ddManager.createProjection (getNextVarOrder (DIR));
    dirIsDir = dirIsDir.eq (ddManager.createProjection (getVarOrder (DIR)));

    // -- next (dir) = up
    MvSet dirIsUp = next (DIR, top);
    // -- next (dir) = down
    MvSet dirIsDown = next (DIR, bot);

    return new CaseStatement ().
      addCase (computeIdle (), dirIsDir).
      addCase (computeFloorEquals (topFloor), dirIsDown).
      addCase (computeFloorEquals (botFloor), dirIsUp).
      addCase (cur (DIR, bot).and (computeCarCallEquals (0)).
	       and (computeLandingCallEquals (0)), 
			dirIsUp).
      addCase (cur (DIR, top).and (computeCarCallEquals (0)).
	       and (computeLandingCallEquals (0)),
			dirIsDown).
      addDefault (dirIsDir).compute ();
  }
  
  MvSet computeFloor ()
  {
    /***
	next (floor) := case
	               door = true : floor;
		       car_call = 0 & landing_call = 0: floor;
		       direction = true & floor < 5 : floor + 1;
		       direction = flase & floor > 1 : floor - 1;
		       1: floor;
		       esac;
    */
     
    MvSet floorIsFloor = computeFloorIsFloor ();
    MvSet floorIsFloorPlusOne = computeFloorIsFloorPlusOne ();
    MvSet floorIsFloorMinusOne = computeFloorIsFloorMinusOne ();
    
    MvSet doorIsOpen = cur (DOOR, top);
    MvSet dirIsDown = cur (DIR, bot);
    MvSet dirIsUp = cur (DIR, top);
    
    return new CaseStatement ().
      addCase (doorIsOpen, floorIsFloor).
      addCase (computeCarCallEquals (0).and 
	       (computeLandingCallEquals (0)), floorIsFloor).
      addCase (dirIsUp.and (computeFloorLess (topFloor)), 
	       floorIsFloorPlusOne).
      addCase (dirIsDown.and (computeFloorGreater (botFloor)), 
	       floorIsFloorMinusOne).
      addDefault (floorIsFloor).
      compute ();
  }

  // -- compute 'floor = v'
  MvSet computeFloorEquals (int v)
  {
    return computeNatEquals (floorBaseIdx, floorWidth, v);
  }
  
  // -- computes 'floor < v'
  MvSet computeFloorLess (int v)
  {
    return computeNatLess (floorBaseIdx, floorWidth, v);
  }
  
  MvSet computeFloorGreater (int v)
  {
    return computeNatGreater (floorBaseIdx, floorWidth, v, topFloor);
  }
  
  MvSet computeDoor ()
  {
    /**
       door := case
                floor = car_call : true;
		floor = landing_call : true;
                1 : false;
               esac
    */

    MvSet curCase = new CaseStatement ().
      addCase (computeFloorIsCarCall (), cur (DOOR, top)).
      addCase (computeFloorIsLandingCall (), cur (DOOR, top)).
      addDefault (cur (DOOR, bot)).compute ();

    // -- we also need to compute the next case for the door, 
    // -- that is
    /* next(door) := case
         next(floor) = next (car_call) : true;
	 1 : false;
	 esac;
    **/
    // -- we acheive this by shifting the variables in the resulting mvset
    curCase = curCase.and (curCase.renameArgs (ordering.getPrimedMap ()));
    return curCase;
    
  }
  
  MvSet computeFloorIsCarCall ()
  {
    return computeFloorIsCall (liftBtns);
  }

  MvSet computeFloorIsLandingCall ()
  {
    // -- treat landing_call as priority_call
    return computeVarIsPriorityCall (floorBaseIdx, floorWidth, topFloor, 
				     main.getLandingBtns ());
  }
  
  MvSet computeFloorIsCall (ButtonR[] btns)
  {
    return computeVarIsCall (floorBaseIdx, floorWidth, topFloor, btns);
  }
  
  MvSet computeEfIsLandingCall ()
  {
    // -- treat landing_call as priority_call
    return computeVarIsPriorityCall (efIdx, efWidth, topFloor, 
				     main.getLandingBtns ());
  }
  
  MvSet computeEfIsCall (ButtonR[] btns)
  {
    return computeVarIsCall (efIdx, efWidth, topFloor, btns);
  }
  MvSet computeVarIsCall (int varIdx, int varWidth, int varTopVal, 
			  ButtonR[] btns)
  {
    MvSet dirIsDown = cur (DIR, bot);
    MvSet dirIsUp = cur (DIR, top);
    
    
    CaseStatement case1 = new CaseStatement ();
    for (int i = topFloor; i > botFloor; i--)
      case1.addCase (btns [i - 1].getPressed ().and
		     (computeNatGreater (varIdx, varWidth, i - 1, varTopVal)),
		     computeNatEquals (varIdx, varWidth, i));
    
    case1.addCase (btns [0].getPressed (), 
		   computeNatEquals (varIdx, varWidth, botFloor));
    case1.addDefault (mvBot);

    
    CaseStatement case2 = new CaseStatement ();
    for (int i = botFloor + 1; i <= topFloor; i++)
      case2.addCase (btns [i - 2].getPressed ().and
		     (computeNatLess (varIdx, varWidth, i)),
		     computeNatEquals (varIdx, varWidth, i - 1));
    case2.addCase (btns [btns.length - 1].getPressed (),
		   computeNatEquals (varIdx, varWidth, topFloor));
    case2.addDefault (mvBot);

    return dirIsDown.and (case1.compute ()).or 
      (dirIsUp.and (case2.compute ()));
    
  }
  

  MvSet computeVarIsPriorityCall (int varIdx, int varWidth, int varTopVal,
				  ButtonR[] btns)
  {
    CaseStatement caseStmt = new CaseStatement ();
    
    // -- compute the case statement
    // -- lift.ef = i & btns[i].pressed : i
    for (int i = 1; i <= topFloor; i++)
      caseStmt.addCase (computeNatEquals (efIdx, efWidth, i).
			and (btns [i-1].getPressed ()),
			computeNatEquals (varIdx, varWidth, i));

    caseStmt.
      addDefault (computeVarIsCall (varIdx, varWidth, varTopVal, btns));

    return caseStmt.compute ();
  }
  

  MvSet computeFloorIsFloor ()
  {
    MvSet result = ddManager.createConstant (bot);
    
    for (int i = botFloor; i <= topFloor; i++)
      result = result.or 
	(computeNatEquals (floorBaseIdx, floorWidth, i).and
	 (computeNextNatEquals (floorBaseIdx, floorWidth, i)));
    return result;
  }
  MvSet computeFloorIsFloorPlusOne ()
  {
    MvSet result = ddManager.createConstant (bot);
    
    for (int i = botFloor; i < topFloor; i++)
      result = result.or
	(computeNatEquals (floorBaseIdx, floorWidth, i).and
	 (computeNextNatEquals (floorBaseIdx, floorWidth, i+1)));
    return result;
  }
  MvSet computeFloorIsFloorMinusOne ()
  {
    MvSet result = ddManager.createConstant (bot);
    for (int i = topFloor; i > botFloor; i--)
      result = result.or
	(computeNatEquals (floorBaseIdx, floorWidth, i).and
	 (computeNextNatEquals (floorBaseIdx, floorWidth, i-1)));
    return result;
  }
  

  

  MvSet computeCarCallEquals (int num)
  {
    return computeCallEquals (num, liftBtns);
  }
  
  MvSet computeLandingCallEquals (int num)
  {
    return computePriorityCallEquals (num, main.getLandingBtns ());
  }
  
  MvSet computePriorityCallEquals (int num, ButtonR[] btns)
  {
    /***
     *** priority_call = 0 iff none of the condition for priority_call hold
     *** and the landing_call = 0
     ***/
    MvSet result = mvBot;

    // -- result is (lift.ef = 1 & landingBut1.pressed) | 
    // --           (lift.ef = 2 & landingBut2.pressed) | ...
    for (int i = 1; i <= topFloor; i++)
      result = result.or 
	(computeNatEquals (efIdx, efWidth, i).
	 and (btns [i-1].getPressed ()));

    result = result.not ();

    return result.and (computeCallEquals (num, btns));
  }
  
  MvSet computeCallEquals (int num, ButtonR[] btns)
  {
    if (num != 0) Assert.assert ("Can only compute 'xxx_call = 0'");
    
    MvSet dirIsDown = cur (DIR, bot);
    MvSet dirIsUp = cur (DIR, top);
    
    MvSet cond1;
    MvSet cond2;

    cond1 = btns [0].getPressed ();
    
    // -- compute a disjunction of 
    // -- butX.PRESSED & floor > (X-1)
    for (int i = botFloor; i < topFloor; i++)
      cond1 = cond1.or 
	(computeFloorGreater (i).and
	 (btns [i].getPressed ()));
    
    
    cond2 = btns [btns.length - 1].getPressed ();

    for (int i = botFloor + 1; i <= topFloor; i++)
      cond2 = cond2.or
	(computeFloorLess (i).and
	 (btns [i-2].getPressed ()));
    

    return dirIsDown.and (cond1.not ()).or (dirIsUp.and (cond2.not ()));
  }
  
  
  /***
   * DEFINE
   * idle := no_call & \A{i} !liftBut_i.pressed
  */
  MvSet computeIdle ()
  {
    MvSet result = noCall;
    
    for (int i = 0; i < liftBtns.length; i++)
      result = result.and (liftBtns [i].getPressed ().not ());
    return result;    
  }

  public MvState initializeInitState (MvState _initState)
  {
    MvState initState = super.initializeInitState (_initState);
    
    initState.setValue (getVarOrder (DOOR), bot);
    initState.setValue (getVarOrder (DIR), bot);
    
    // -- set 'init (floor) = 1'
    // -- set 'init (ef) = 0'
    initState.setValue (getVarOrder (floorBaseIdx), top);
    initState.setValue (getVarOrder (efIdx), bot);
    
    for (int i = 1; i < floorWidth; i++)
      {
	initState.setValue (getVarOrder (floorBaseIdx + i), bot);
	initState.setValue (getVarOrder (efIdx + i), bot);
      }
      
    return initState;
  }

}




