package edu.toronto.cs.mdd;

import java.util.*;
import java.io.*;
import edu.toronto.cs.modelchecker.*;

/*** 
 *** Converts a Kripke structure to daVinci format
 ***/
public class MDDNodeToDaVinci
{
  Map nodes;
  
  MDDNode mdd;
  PropOrderIf ordering;
  
  
  
  static Map colorMap;

  static 
  {
        // -- XXX Shameless hack
    colorMap = new HashMap ();
    colorMap.put ("T", "black");
    colorMap.put ("M", "blue");
    colorMap.put ("S", "green");
    colorMap.put ("DN", "orange");
    colorMap.put ("DK", "orange");
    colorMap.put ("DC", "red");
    colorMap.put ("SN", "purple");
    colorMap.put ("F", "grey");

    colorMap.put ("0", "black");
    colorMap.put ("1", "red");
    colorMap.put ("2", "green");
    colorMap.put ("3", "blue");
  }
  
  
  public static Map getColorMap ()
  {
    return colorMap;
  }

  public MDDNodeToDaVinci (MDDNode _mdd)
  {
    this (_mdd, null);
  }
  
  public MDDNodeToDaVinci (MDDNode _mdd, PropOrderIf _ordering)
  {
    mdd = _mdd;
    ordering = _ordering;
    nodes = new HashMap ();
  }

  public static void mddToDaVinci (MDDNode node, String fileName)
  {
    mddToDaVinci (node, null, fileName);
  }
  
  public static void mddToDaVinci (MDDNode node, PropOrderIf ordering, 
				   String fileName)
  {
    try {
      PrintWriter out = new PrintWriter (new FileWriter (fileName));
      
      MDDNodeToDaVinci converter = new MDDNodeToDaVinci (node, ordering);
      converter.computeGraph ();
      converter.dump (out);
      out.flush ();
    }
    catch (Exception ex)
      {
	ex.printStackTrace ();
      }
  }
  

  public void computeGraph ()
  {
    computeGraph (mdd);
  }

  public void computeGraph (MDDNode mdd)
  {
    String mddName = getMDDName (mdd);
    String mddValue = getMDDValue (mdd);
    if (nodes.containsKey (mddName))
      return;

    Node node = new Node (mddName, mddValue);
    nodes.put (mddName, node);

    edu.toronto.cs.algebra.Algebra algebra = mdd.getManager ().getAlgebra ();
    for (int i = 0; i < mdd.howManyChildren (); i++)
      {
	node.addEdge (getMDDName (mdd.getChild (i)), 
		      algebra.getElementKeyByIdx (i).toString ());
	computeGraph (mdd.getChild (i));
      }
    
  }

  Map mddNames = new HashMap ();
  int nameIdx = 0;
  
  public String getMDDName (MDDNode mdd)
  {
    String name = (String)mddNames.get (mdd);
    if (name == null)
      {
	name = "s_" + String.valueOf (nameIdx++);
	mddNames.put (mdd, name);
      }
    
    return name;
  }
  
  public String getMDDValue (MDDNode mdd)
  {
    if (mdd.isConstant ())
      // -- expand the index into human readable form
      return mdd.getValueKey ().toString ();    
    else
      return getVarName (mdd.getVarIndex ());
  }

  String getVarName (int idx)
  {
    if (ordering == null) return "x_" + idx;
    
    String name = ordering.getPropName (idx);
    if (ordering.isPostImage (idx))
      name = name + "'";
    return name;  
  }  
  
  public void dump (PrintWriter out)
  {
    out.println ("[");
    for (Iterator it = nodes.values ().iterator (); it.hasNext ();)
      {
	out.print (it.next ());
	if (it.hasNext ())
	  out.println (",");
      }
    out.println ("]");
  }

  public static String quote (String s)
  {
    return "\"" + s + "\"";
  }
  
  
  public static class Node
  {
    String nodeData;
    String name;
    List edges;
    String color;
    
    public Node (String _name, String _nodeData)
    {
      name = _name;
      nodeData = _nodeData;
      edges = new ArrayList ();

      // -- set the color if possible
      color = (String)getColorMap ().get (nodeData);
      if (color == null)
	color = "black";

    }
    
    public void addEdge (String dst, String value)
    {
      String color = (String)getColorMap ().get (value);

      if (color == null) 
	{
	  System.out.println ("No color for: " + value);
	  color = "black";
	}
      
      edges.add (new Edge (name, dst, value, color));
    }
    
    public String toString ()
    {
      StringBuffer sb = new StringBuffer ();
      sb.append ("l(" + quote (name));
      sb.append (", n(" + quote ("State"));
      sb.append (", [a(" + quote ("OBJECT") + ", " + quote (nodeData) + ")");
//        sb.append (", a(" + quote ("_GO") + ", " + quote ("text") + ")");
//        sb.append (", a(" + quote ("COLOR") + ", " + quote (color) + ")");
      sb.append ("]");
      sb.append (", [");
      for (Iterator it = edges.iterator (); it.hasNext ();)
	{
	  sb.append (it.next ().toString ());
	  if (it.hasNext ())
	    sb.append (",");
	}
      sb.append ("]");
      sb.append ("))");
      return sb.toString ();
    }
    
  }
  
  public static class Edge
  {
    // -- name of the destanation and source nodes
    String src;
    String dst;
    String color = "black";
    String value;
    
    public Edge (String _src, String _dst, String _value, String _color)
    {
      src = _src;
      dst = _dst;
      value = _value;
      color = _color;
    }
    public Edge (String src, String dst, String value)
    {
      this (src, dst, value, "black");
    }
    
    public String toString()
    {
      StringBuffer sb = new StringBuffer ();
      sb.append ("l(" + quote (src + "->" + dst + "_" + value));
      sb.append (", e(" + quote ("Edge"));
      sb.append (", [a(" + quote ("EDGECOLOR") + "," + quote (color) + ")]");
      sb.append (",r(" + quote (dst) + ")))");
      return sb.toString ();
    }
    
  }
  
}
