package edu.toronto.cs.modelchecker;

import edu.toronto.cs.util.*;
import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.algebra.*;
import edu.toronto.cs.mvset.*;
import edu.toronto.cs.smv.*;

import java.util.*;
import java.lang.reflect.Method;


/**
 ** A reader for SMV models.
 **/
public class SMVModelReader extends AbstractModelReader
{
    // initialize the logger
  static Logger log = new Logger (SMVModelReader.class.getName ());

  // the SMV module
  SMVModule smvModule;

  public SMVModelReader (Map opts) throws Exception
  {
    // -- here we initialize the model checker using SMV module from 
    // -- command line argument. 
    // -- we need to initialize
    // -- 1) Lattice
    // -- 2) Prop ordering
    
    options = opts;
    List cmdLine = (List) options.get ("input");

    // -- extract smvModule name and its arguments
    String smvModuleName = (String)cmdLine.get (0);
    String[] smvModuleArgs = (String[])cmdLine.subList (1, cmdLine.size ()).
      toArray (new String [cmdLine.size () - 1]);
    
    smvModule = loadSMVModule (smvModuleName, smvModuleArgs);
    // set up the default model name
    options.put ("model name", smvModule.getModelName ());

    propOrder = new AlternatePropOrder (smvModule.getVars ());
    
    smvModule.setVarOrdering (propOrder);
    System.out.println ("varsLength is now: " + smvModule.getVars().length);
  }  


  /**
   ** Given an MvSetFactory creates a transition relation.
   **/
  public void makeTransRelation (MvSetFactory ddManager)
  {
    // -- 3) Transition relation
    // -- 4) Initial state

    FiniteLattice lattice = smvModule.getLattice ();
    
    smvModule.setMvSetFactory (ddManager);

    // -- set init state
    initState = new MvState (propOrder.size (), lattice);
    initState = smvModule.initializeInitState (initState);

    // -- enumerated types side-effect
    enumTypes = smvModule.getEnumTypes ();

    // -- compute transition relation
    transRelation = smvModule.computeTrans ();
  }

  private SMVModule loadSMVModule (String className, String[] args) 
    throws Exception
  {
    Class clazz = Class.forName (className);
    Method initMethod = clazz.getMethod ("initModule", 
					 new Class[]{String[].class});
    return (SMVModule)initMethod.invoke (clazz, new Object[]{args});
  }

  /**
   ** Get the lattice.
   ** @return lattice.
   **/
  public FiniteLattice getLattice ()
  {
    return smvModule.getLattice ();
  }

}


