package edu.toronto.cs.prettygraph;

import java.util.*;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.mvset.MvState;


public class DVGenNodeVisitor extends PrettyGraphVisitorAncestor 
  implements PrettyGraphVisitor
{
  static Logger log = new Logger ("DVGenNodeVisitor");

  Set dv;
  DVGenEdgeVisitor prototypeEdger;
  Set extraStack;
  Set nodeNames;
  Map nodes;

  public DVGenNodeVisitor (DVGenEdgeVisitor _edger)
  {
    super ();
    prototypeEdger = _edger;
    Assert.assert (prototypeEdger != null, "edger null!");
    nodes = new HashMap ();
    nodeNames = new HashSet ();
    //extraStack = new StringBuffer ();
    dv = new HashSet ();
  }

  public StringBuffer getSB ()
  {
    return Dv.enum(dv);
  }

  public void visitPrettyGraphNode (PrettyGraphNode node, Object stateinfo)
  {
    log.debug ("Hi -- visitPrettyGraphNode.");
    
    extraStack = new HashSet ();
    if (!nodes.containsKey (node.getName ()))
      {
	StringBuffer dvnode = Dv.item (""+node.getName (), 
				     Dv.nodebody 
				     (node.getType (),
				      Dv.enum (doAttributes (node)), 
				      Dv.enum (getEdges (node))));
	Assert.assert 
	  (!nodeNames.contains(node.getName ()), "Arg! Namespace clash!");
	nodeNames.add (node.getName ());
	nodes.put (node.getName (), dvnode);
      }
    log.debug ("We are now trying to retreive: " + node.getName());
    dv.add (nodes.get (node.getName ()));
    for (Iterator it = extraStack.iterator (); it.hasNext ();)
      {
	dv.add (it.next ());
      }
  }

  public void visitPrettyKegGraphNode (PrettyKegGraphNode node, 
				       Object stateinfo)
  {
    log.debug ("Visit prettykeggraphnode");
    visitPrettyGraphNode (node, stateinfo);
  }
  
  public void visitPrettyGraphMDDNode (PrettyGraphMDDNode node, 
				       Object stateinfo)
  {
    visitPrettyGraphNode (node, stateinfo);
  }

  public void visitPrettyXKNode (PrettyGraphXKNode node,
				 Object stateinfo)
  {
    visitPrettyGraphNode (node, stateinfo);
  }
  
  protected Collection doAttributes (PrettyGraphNode node)
  {
    Set result = new HashSet ();
    result = doLabel (result, node);
    result = doColour (result, node);
    result = doTextStyle (result, node);
    result = doBoxType (result, node);
    result = doLineType (result, node);
    result = doHid (result, node);
    log.debug ("Attributes: "+result);
    return result;
  }
    
  protected Set doHid (Set result, PrettyGraphNode node)
  {
    if (node.getHiddenToStart ())
      result.add (Dv.attrib ("HIDDEN", "true"));
    return result;
  }
  
  
  protected Set doLabel (Set result, PrettyGraphNode node)
  {
    StringBuffer sb = Dv.attrib ("OBJECT", ""+node.getLabel ());
    result.add(sb);
    return result;
  }
    
  protected Set doColour (Set result, PrettyGraphNode node)
  {
    if (node.getColour () != null)
      result.add (Dv.attrib ("COLOR", Dv.translateColour (node.getColour ())));
    return result;
  }
  
  protected Set doTextStyle (Set result, PrettyGraphNode node)
  {
    if (node.getFontFamily ()!= "")
      result.add (Dv.attrib("FONTFAMILY", node.getFontFamily ()));
    if (node.getFontStyle ()!= "")
      result.add (Dv.attrib("FONTSTYLE", node.getFontStyle ()));
    return result;
  }
  
  protected Set doBoxType (Set result, PrettyGraphNode node)
  {
    if (node.getShape () != 0)
      {
      if (node.getShape () == PrettyConstants.BOX_SHAPE)
	result.add (Dv.attrib ("_GO", "box"));
      else if (node.getShape () == PrettyConstants.CIRCLE_SHAPE)
	result.add (Dv.attrib ("_GO", "circle"));
      }
    return result;
  }
  
  protected Set doLineType (Set result, PrettyGraphNode node)
  {
    String attribute = "BORDER";
    if (node.getLineNum ()!= 0)
      {
	if (node.getLineNum () == PrettyConstants.SINGLE_LINE)
	  {
	    result.add (Dv.attrib (attribute, "single"));
	    return result;
	  }
	else if (node.getLineNum () == PrettyConstants.DOUBLE_LINE)
	  {
	    result.add (Dv.attrib (attribute, "double"));
	    return result;
	  }
      }
    if (node.getLineThickness ()!= 0)
      {
	if (node.getLineThickness ()== PrettyConstants.THIN_LINE)
	  {
	    result.add (Dv.attrib (attribute, "thin"));
	    return result;
	  }
	else if (node.getLineThickness ()== PrettyConstants.THICK_LINE)
	  {
	    result.add (Dv.attrib (attribute, "thick"));
	    return result;
	  }
      }
    if (node.getLineSolidity () != 0)
      {
	if (node.getLineSolidity () == PrettyConstants.NO_LINE)
	  {
	    result.add (Dv.attrib (attribute, "none"));
	    return result;
	  }
	// So far as I know, dv doesn't support dotted borders...
      }
    return result;
    
  }
  
  protected Collection getEdges (PrettyGraphNode node)
  {
    log.debug ("Hi -- getting edges.");
    Set result = new HashSet ();
    DVGenEdgeVisitor edger = (DVGenEdgeVisitor)prototypeEdger.clone ();
    edger.setSuperNodeLabel (""+node.getName ());
    for (Iterator it = node.getAllEdges ().iterator ();
	 it.hasNext ();)
      {
	edger.doEdge ((DGEdge)it.next (), null);
      }
    for (Iterator it = edger.getEdges ().iterator (); it.hasNext ();)
      {
	DVEdge edge = (DVEdge) it.next ();
	result.add (edge.getEdge ());
	if (edge.getExtraState () != null)
	  {
	    extraStack.add (edge.getExtraState ());
	    Assert.assert (!nodeNames.contains (edge.getExtraStateName ()), 
	       "Arg Namespace clash in getEdges: "+edge.getExtraState () 
	       + " --- \n " + extraStack);
	    nodeNames.add (edge.getExtraStateName ());
	  }
      }
    return result;
  }
}

    
	

    






