package edu.toronto.cs.prettygraph;

import java.util.*;

import edu.toronto.cs.xkripke.*;
import edu.toronto.cs.util.dgraph.*;
import edu.toronto.cs.util.*;
import edu.toronto.cs.modelchecker.*;
import edu.toronto.cs.keg.*;
import edu.toronto.cs.mvset.MvState;


public class PrettyKegColouringStrategy extends PrettyColouringStrategy
{

  static Logger log = new Logger ("PrettyKegColouringStrategy");
  
  public static Object CTL_COLOUR = new java.awt.Color (176, 196, 222);
  public static String CTL_FONTSTYLE = "italic";
  public static String CTL_FONTFAMILY = "courier";
  public static boolean CTL_HIDTOSTART = true;
  public static int CTL_BORDERLINENUM = 0;
  public static int CTL_BORDERLINESOLIDITY = PrettyConstants.NO_LINE;
  public static String CTL_GRAPHIC = "box";
  
  public static Object STATE_COLOUR = new java.awt.Color (255, 255, 255);
  public static String STATE_FONTSTYLE = "bold";
  public static String STATE_FONTFAMILY = "times";
  public static boolean STATE_HIDTOSTART = false;
  public static int STATE_BORDERLINENUM = PrettyConstants.DOUBLE_LINE;
  public static int STATE_BORDERLINESOLIDITY = PrettyConstants.SOLID_LINE;
  public static String STATE_GRAPHIC = "";


  PrettyKegEdgeColouring edger;
  PrettyStateLabelStrategy stateLabellingStrategy;
  PrettyCtlLabelStrategy ctlLabellingStrategy;
  PropOrderIf prop;

  public PrettyKegColouringStrategy (PropOrderIf _prop)
  {
    super ();
    log.debug ("Initialising PrettyKegColouringStrategy.");
    edger = null;
    ctlLabellingStrategy = new PrettySensibleCtlLabelStrategy ();
    prop = _prop;
  }
  
  public PrettyKegColouringStrategy setEdger (PrettyKegEdgeColouring _edger)
  {
    edger = _edger;
    return this;
  }
  
  public PrettyKegColouringStrategy setLabeller 
    (PrettyStateLabelStrategy _labeller)
  {
    stateLabellingStrategy = _labeller;
    return this;
  }

  public void colourCtl (KEGCtlNode ctl, PrettyKegGraphNode node, 
			 Object stateinfo)
  {
    // First off, the node itself:
    List ancestry = (List) stateinfo;
    //int i;
    //int j = 1;
    PrettyKegGraphNode knode = null;
    //System.out.println ("Dealing with node: "+ctl);
    //System.out.println ("Ancestry size: "+ancestry.size ());
    for (int i = ancestry.size (); i > 0; i--)
      {
	//System.out.println ("   On node: "+ancestry.get (i-1));
	if (((PrettyKegGraphNode)ancestry.get (i-1)).getKegNode () instanceof KEGStateNode)
	  {
	    knode = (PrettyKegGraphNode) ancestry.get (i-1);
	    break;
	  }
      }
    Assert.assert (knode != null);
    // PrettyKegGraphNode knode = (PrettyKegGraphNode) ancestry.get (j-1);
    node.setLabel (getLabelFor (ctl, stateinfo, knode))
      .setColour (CTL_COLOUR)
      .setFontStyle (CTL_FONTSTYLE)
      .setFontFamily (CTL_FONTFAMILY)
      .setHiddenToStart (CTL_HIDTOSTART)
      .setLineNum (CTL_BORDERLINENUM)
      .setLineSolidity (CTL_BORDERLINESOLIDITY)
      .setGraphicName (CTL_GRAPHIC);
    // Now, all the transitions.
    if (node.getAllEdges ()!=null)
      {
	log.debug ("Yes! We're colouring an edge...");
	for (Iterator it = node.getAllEdges ().iterator (); it.hasNext ();)
	  edger.visit ((DGEdge)it.next (), null);
      }
  }
  
  public void colourState (KEGStateNode state, PrettyKegGraphNode node, Object stateinfo)
  {
    //node.setLabel (state.getState ().toString (prop))
    node.setLabel (getLabelFor (state, stateinfo))
      .setColour (STATE_COLOUR)
      .setFontStyle (STATE_FONTSTYLE)
      .setFontFamily (STATE_FONTFAMILY)
      .setHiddenToStart (STATE_HIDTOSTART)
      .setLineNum (STATE_BORDERLINENUM)
      .setLineSolidity (STATE_BORDERLINESOLIDITY)
      .setGraphicName (STATE_GRAPHIC);
    if (state.getAllEdges ()!=null)
      {
	log.debug ("Yes! We're colouring an edge...");
	for (Iterator it = node.getAllEdges ().iterator (); it.hasNext ();)
	  {
	    DGEdge edge = (DGEdge)it.next ();
	    log.debug (" Edge "+edge);
	    edger.visit ((DGEdge)edge, null);
	  }
	
      }
    
  }

  public void visitPrettyKegGraphNode (PrettyKegGraphNode node, 
				       Object stateinfo)
  {
    // First off, what kind of node are we dealing with here?
    String type = node.getType ();
    log.debug ("Colouring node "+node.getName ()+" of type "+type);
    DGNode kegnode = node.getKegNode ();
    log.debug ("Of keg "+kegnode);
    if (type == PrettyConstants.ST_TYPE)
      colourState ((KEGStateNode)kegnode, node, stateinfo);
    else if (type == PrettyConstants.CTL_TYPE)
      colourCtl ((KEGCtlNode)kegnode, node, stateinfo);
  }

  private Object getLabelFor (KEGStateNode statenode, Object stateinfo)
  {
    return stateLabellingStrategy.getLabel (statenode, stateinfo);
  }

  private Object getLabelFor (KEGCtlNode statenode, Object stateinfo, PrettyKegGraphNode ownerState)
  {
    ctlLabellingStrategy.setOwnerState 
      (((KEGStateNode)ownerState.getKegNode ()));
    return ctlLabellingStrategy.getLabel (statenode, stateinfo);
  }
  

}











