package edu.toronto.cs.smv;

import java.util.*;
import edu.toronto.cs.util.ArrayUtil;
import edu.toronto.cs.util.userprompt.*;


public class SimpleEnumTypes 
   implements EnumeratedTypes 
{
  
  Map rangeMaps;
  
  static int bitsForRange(int n) 
  {
    return (int)Math.ceil(Math.log((double)n)/Math.log(2.0));
 
  }
  
  static class VarEntry 
  {
    public String[] decodings;
    public Map encodings;
    public String[] range;
    
    public VarEntry(String[] rg) 
    {
      range = rg;
      
      decodings = new String[2 << bitsForRange(rg.length)];
      encodings = new HashMap(2 << bitsForRange(rg.length));

      for (int i=0; i<rg.length; i++)
	encodings.put(rg[i], new ArrayList(bitsForRange(rg.length)));
    }
    
    public void xEncodesV(int x, String v) 
    {
      decodings[x] = v;
      ((ArrayList)encodings.get(v)).add(new Integer(x));
    }
    
  };
  

  public boolean isEnumeratedVariable(String varName) 
  {
    return rangeMaps.containsKey(varName);
    
    
  }
  
  public int getRangeSize(String varName) 
  {
    if (isEnumeratedVariable(varName))
      return ((VarEntry)(rangeMaps.get(varName))).range.length;
    else return 2;
  }

  public int getRangeBits(String varName) 
  {
    return bitsForRange(getRangeSize(varName));
    
  }
  
  public boolean variableHasValue(String varName, String valName)
  {
    return ((VarEntry)(rangeMaps.get(varName))).encodings.containsKey(valName);
    
  }
  
  public String valueEncoded(String varName, int encoding) 
  {
    return ((VarEntry)(rangeMaps.get(varName))).decodings[encoding];
    
    
    
  }
  
  public int getEncoding(String varName, String valName) 
  {
    ArrayList encList= ( (ArrayList)
			 (((VarEntry)(rangeMaps.get(varName))).encodings.get(valName)) );
    return ( (Integer)(encList.get(0))).intValue();
  }

  public int[] getAllEncodings(String varName, String valName) 
  {
    ArrayList encList= ( (ArrayList)
			 (((VarEntry)(rangeMaps.get(varName))).encodings.get(valName)) );
    
    Integer[] encs = (Integer []) encList.toArray(new Integer[encList.size()]);
    int[] retVal = new int[encs.length];
   
    for (int i=0; i<encs.length; i++) 
      
  
      retVal[i] = encs[i].intValue();
    
    return retVal;
    
    
  }
  
  
  public SimpleEnumTypes() 
  {
    rangeMaps = new HashMap();

  }
  
  public void addVariable(String vName, String[] range) 
  {
    VarEntry nve = new VarEntry(range);
    rangeMaps.put(vName, nve);
    encodeRangeRecurInts(range, 0, bitsForRange(range.length), 0, nve);
    
  }

 
  public static void main(String argv[]) 
  {
    SimpleEnumTypes seti = new SimpleEnumTypes();
    
    seti.addVariable("foobar", argv);
    NiceReader ni = new NiceReader(System.in);
    
    while (true) {
      System.out.println("(E)ncode or (D)ecode?");
      String choice = ni.readLine();
      if (choice.equals("D"))
	{
	  System.out.println("What number?");
	  int num = Integer.parseInt(ni.readLine());
	  System.out.println("Encodes for: "+seti.valueEncoded("foobar", num));
	  
	  
	}
      else
	{
	  System.out.println("What symbol?");
	  String sym = ni.readLine();
	  
	  System.out.println(ArrayUtil.toString(seti.getAllEncodings("foobar", sym)));
	  
	  
	}
      
      
      
      
    }
    
    
  }
  

  static void encodeRangeRecurInts(String[] range, int currLevel, int maxLevels,
			int enco, VarEntry ve)
  {
    if (range.length == 1)
      {
	if (currLevel == maxLevels)
	  //	  System.out.println(enco+ " encodes "+range[0]);
	  ve.xEncodesV(enco, range[0]);
	
	else // currlevel < maxLevels
	  {
	    encodeRangeRecurInts(range, currLevel+1, maxLevels, 
				 enco, ve);
	    encodeRangeRecurInts(range, currLevel+1, maxLevels, 
				 enco | (1 << currLevel), ve);
	    
	    
	  }
      }
    else // range.length > 1
      {
	String[] evens = ArrayUtil.evenElements(range);
	String[] odds =  ArrayUtil.oddElements(range);
	encodeRangeRecurInts(evens, currLevel+1, maxLevels, enco,ve);
	encodeRangeRecurInts(odds, currLevel+1, maxLevels, 
			     enco | (1 << currLevel) ,ve);
      }
 
    
  } // end encodeRangeRecur
  

  public List getRange(String varName) 
  {
    return Arrays.asList(((VarEntry)(rangeMaps.get(varName))).range);
    
    
  }
  
    
}

